function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var styles = {"row_cell":"_3TtXw","row_cell_invisible":"_2fzm0","row":"_32f2G","row_invisible":"_kbpmo"};

function makeId(length) {
  if (length === void 0) {
    length = 15;
  }

  var text = '';
  var possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

  for (var i = 0; i < length; i++) {
    text += possible.charAt(Math.floor(Math.random() * possible.length));
  }

  return text;
}
var isBetween = function isBetween(min, max, num) {
  var states = num >= min && max >= num;

  if (states) {
    console.log(min, max, num);
  }

  return states;
};
function validForBar(number) {
  return !(!Number.isInteger(number) || number < 0);
}
function evaluatePosition(firstPosition, secondPosition) {
  if (!validForBar(firstPosition.column)) {
    throw new Error("firstPosition.column expected an integer received " + firstPosition.column);
  }

  if (!validForBar(firstPosition.row)) {
    throw new Error("firstPosition.row expected an integer received " + firstPosition.row);
  }

  if (!validForBar(secondPosition.column)) {
    throw new Error("secondPosition.column expected an integer received " + secondPosition.column);
  }

  return {
    row: firstPosition.row,
    column: firstPosition.column,
    length: secondPosition.column - firstPosition.column + 1
  };
}
function getPosition(row, column, dimension, leftTitleWidth, topTitleHeight) {
  if (leftTitleWidth === void 0) {
    leftTitleWidth = 0;
  }

  if (topTitleHeight === void 0) {
    topTitleHeight = 0;
  }

  return {
    left: leftTitleWidth + column * dimension,
    top: row * dimension + topTitleHeight
  };
}
function getColumnCount(dimension, width, rowTitleWidth) {
  return Math.floor((width - rowTitleWidth) / dimension);
}
function getRowCount(dimension, height) {
  return Math.floor(height / dimension);
}
function resizeBar(bars, newLocation) {
  return bars.map(function (bar) {
    if (bar.editing) {
      var nPosition = evaluatePosition({
        column: bar.column,
        row: bar.row
      }, newLocation.cell);
      return _extends({}, bar, nPosition);
    }

    return bar;
  });
}

var actionTypes = {
  add: 'ADD',
  edit: 'EDIT',
  "delete": 'DELETE',
  addToEditing: 'ADD_TO_EDITING',
  doneEditing: 'DONE_EDITING',
  setBars: 'SET_BARS'
};

function checkCollision(bars, eBar) {
  var editingBar = _extends({}, eBar);

  var oBars = bars.map(function (b) {
    if (editingBar.row === b.row) {
      var bStart = b.column + 1;
      var barStart = editingBar.column + 1;
      var bTotal = b.column + b.length;
      var editingBarTotal = editingBar.column + editingBar.length;

      if (isBetween(bStart, bTotal, barStart) || isBetween(bStart, bTotal, editingBarTotal) || isBetween(barStart, editingBarTotal, bTotal) || isBetween(barStart, editingBarTotal, bStart)) {
        var _collided = collided(b, editingBar),
            bar1 = _collided[0],
            bar2 = _collided[1];

        editingBar = bar2;
        return bar1;
      }
    } else {
      var _removeCollision = removeCollision(b, editingBar),
          _bar = _removeCollision[0],
          _bar2 = _removeCollision[1];

      editingBar = _bar2;
      return _bar;
    }

    return b;
  });
  return [oBars, editingBar];
}

var collided = function collided(bar1, bar2) {
  bar1 = checkHasCollisionObject(bar1);
  bar1.collisions[bar2.id] = '';
  bar2 = checkHasCollisionObject(bar2);
  bar2.collisions[bar1.id] = '';
  return [bar1, bar2];
};

var checkHasCollisionObject = function checkHasCollisionObject(bar) {
  if (!bar.collisions) {
    bar.collisions = {};
  }

  return bar;
};

var removeCollision = function removeCollision(bar1, bar2) {
  bar1 = checkHasCollisionObject(bar1);
  delete bar1.collisions[bar2.id];
  bar2 = checkHasCollisionObject(bar2);
  delete bar2.collisions[bar1.id];
  return [bar1, bar2];
};

function reserverReducer(state, action) {
  switch (action.type) {
    case actionTypes.edit:
      {
        var nBars = [].concat(state.bars);
        var bIndex = nBars.findIndex(function (bar) {
          return action.payload.id === bar.id;
        });
        var otherBars = [].concat(nBars);
        otherBars.splice(bIndex, 1);

        var _checkCollision = checkCollision(otherBars, action.payload),
            oBars = _checkCollision[0],
            editingBar = _checkCollision[1];

        oBars.push(editingBar);
        return _extends({}, state, {
          bars: oBars
        });
      }

    case actionTypes.add:
      {
        var bars = [].concat(state.bars);
        bars.push(action.payload);
        return _extends({}, state, {
          bars: bars,
          isEditing: true
        });
      }

    case actionTypes.setBars:
      {
        return _extends({}, state, {
          bars: action.payload
        });
      }

    case actionTypes["delete"]:
      {
        var _nBars = [].concat(state.bars);

        var _bIndex = _nBars.findIndex(function (bar) {
          return action.payload.id === bar.id;
        });

        _nBars.splice(_bIndex, 1);

        return _extends({}, state, {
          bars: _nBars
        });
      }

    case actionTypes.doneEditing:
      {
        if (state.isEditing) {
          var _nBars2 = [].concat(state.bars);

          _nBars2 = _nBars2.map(function (bar) {
            if (bar.editing) {
              var barStyle = _extends({}, bar.style);

              barStyle.pointerEvents = 'auto';
              return _extends({}, bar, {
                editing: false,
                style: barStyle
              });
            }

            return bar;
          });
          return _extends({}, state, {
            bars: _nBars2,
            isEditing: false
          });
        }

        return state;
      }

    case actionTypes.mouseDragOver:
      {
        return _extends({}, state);
      }

    default:
      {
        console.log("Unhandled type: " + action.type);
        return state;
      }
  }
}

function useReserver(reducer, initialState) {
  var _useReducer = React.useReducer(reducer, {
    bars: initialState,
    isEditing: false
  }),
      _useReducer$ = _useReducer[0],
      bars = _useReducer$.bars,
      isEditing = _useReducer$.isEditing,
      dispatch = _useReducer[1];

  var addBar = function addBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes.add
    });
  };

  var editBar = function editBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes.edit
    });
  };

  var deleteBar = function deleteBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes["delete"]
    });
  };

  var setBars = function setBars(props) {
    console.log(props);
    return dispatch({
      payload: props,
      type: actionTypes.setBars
    });
  };

  var setIsEditing = function setIsEditing(props) {
    return dispatch({
      payload: props,
      type: actionTypes.setEditing
    });
  };

  return {
    isEditing: isEditing,
    setIsEditing: setIsEditing,
    bars: bars,
    addBar: addBar,
    editBar: editBar,
    deleteBar: deleteBar,
    setBars: setBars
  };
}

function Bar(props) {
  return /*#__PURE__*/React__default.createElement("div", {
    role: "listitem",
    onDragStart: props.onDragStart,
    onClick: function onClick(e) {
      props.onClick(e, props);
    },
    onMouseOver: function onMouseOver(e) {
      props.onMouseOver(e, props);
    },
    onContextMenu: function onContextMenu(e) {
      props.onContextMenu(e);
    },
    draggable: props.draggable,
    style: _extends({}, props.style, {
      pointerEvents: props.style.pointerEvents || 'none',
      background: props.style.background || '#0E6BA8',
      display: 'flex',
      position: 'absolute',
      zIndex: props.zIndex || '100'
    }),
    className: props.className
  }, [].concat(Array(props.length && props.length > 0 ? props.length : 1)).map(function (notUsed, i) {
    return /*#__PURE__*/React__default.createElement("div", {
      key: i,
      style: {
        width: props.dimension,
        height: props.dimension,
        pointerEvents: props.style.pointerEvents || 'none'
      }
    });
  }), props.children);
}
Bar.defaultProps = {
  style: {},
  dimension: 20,
  onClick: function onClick() {},
  onMouseOver: function onMouseOver() {},
  onDragStart: function onDragStart() {},
  onContextMenu: function onContextMenu() {},
  length: 1
};

function Head(props) {
  var _useState = React.useState([]),
      headRow = _useState[0],
      setHeadRow = _useState[1];

  React.useEffect(function () {
    if (typeof props.headRow === 'function') {
      setHeadRow(props.headRow(props.columnCount));
    } else if (Array.isArray(props.headRow)) {
      setHeadRow(props.headRow);
    }
  }, [props.headRow, props.columnCount]);
  return /*#__PURE__*/React__default.createElement("div", {
    role: "columnheader",
    className: headRow.length > 0 ? styles.row : styles.row_invisible,
    style: {
      height: props.dimension
    }
  }, /*#__PURE__*/React__default.createElement("div", {
    className: styles.row_cell,
    style: {
      width: props.rowTitleWidth,
      height: props.dimension + 'px'
    }
  }), headRow.map(function (headitem, i) {
    return /*#__PURE__*/React__default.createElement("div", {
      "aria-colindex": i,
      key: i,
      style: {
        width: props.dimension + 'px',
        height: props.dimension + 'px'
      },
      className: styles.row_cell
    }, headitem);
  }));
}

function Reserver(props) {
  var _useState = React.useState([]),
      rowTitles = _useState[0],
      setRowTitles = _useState[1];

  var _useState2 = React.useState(0),
      rowCount = _useState2[0],
      setRowCount = _useState2[1];

  var _useState3 = React.useState(0),
      columnCount = _useState3[0],
      setColumnCount = _useState3[1];

  React.useEffect(function () {
    setColumnCount(getColumnCount(props.dimension, props.width, props.rowTitleWidth));
  }, [props.width, props.dimension]);
  React.useEffect(function () {
    setRowCount(getRowCount(props.dimension, props.height));
  }, [props.height, props.dimension]);
  React.useEffect(function () {
    if (typeof props.rowTitles === 'function') {
      setRowTitles(props.rowTitles());
    } else if (Array.isArray(props.rowTitles)) {
      setRowTitles(props.rowTitles);
    }
  }, [props.rowTitles]);
  return /*#__PURE__*/React__default.createElement("div", {
    role: "grid",
    onMouseLeave: props.mouseLeaveGrid,
    style: _extends({}, props.style, {
      position: 'relative'
    })
  }, /*#__PURE__*/React__default.createElement(Head, {
    headRow: props.headRow,
    columnCount: columnCount,
    rowTitleWidth: props.rowTitleWidth,
    dimension: props.dimension
  }), [].concat(Array(rowCount)).map(function (x, r) {
    return /*#__PURE__*/React__default.createElement("div", {
      role: "rowgroup",
      className: styles.row,
      key: r,
      style: {
        height: props.dimension,
        display: 'flex'
      }
    }, /*#__PURE__*/React__default.createElement("div", {
      className: rowTitles.length > 0 ? styles.row_cell : styles.row_cell_invisible,
      style: {
        width: props.rowTitleWidth,
        height: props.dimension + 'px'
      }
    }, rowTitles[r]), [].concat(Array(columnCount)).map(function (x, c) {
      return /*#__PURE__*/React__default.createElement("div", {
        role: "gridcell",
        "aria-colindex": c,
        onDragOver: function onDragOver(e) {
          e.preventDefault();
          props.mouseDragOverCell({
            cell: {
              row: r,
              column: c
            }
          }, e);
        },
        className: styles.row_cell,
        onMouseEnter: function onMouseEnter(e) {
          props.mouseEnterCell({
            dimension: props.dimension,
            cell: {
              row: r,
              column: c
            }
          }, e);
        },
        onMouseDown: function onMouseDown(e) {
          props.mouseDownCell({
            dimension: props.dimension,
            cell: {
              row: r,
              column: c
            }
          }, e);
        },
        onMouseUp: function onMouseUp(e) {
          props.mouseUpCell({
            cell: {
              row: r,
              column: c
            }
          }, e);
        },
        onDrop: function onDrop(e) {
          props.mouseCellDrop({
            cell: {
              row: r,
              column: c
            }
          }, e);
        },
        style: {
          width: props.dimension + 'px',
          height: props.dimension + 'px'
        },
        key: c
      }, props.content["r" + r + "c" + c]);
    }));
  }), /*#__PURE__*/React__default.createElement("div", {
    role: "list"
  }, typeof props.children === 'function' && props.children({
    rowCount: rowCount,
    columnCount: columnCount,
    rowTitleWidth: props.rowTitleWidth,
    dimension: props.dimension,
    rowTitleHeight: rowTitles.length > 0 ? props.dimension : 0
  }), Array.isArray(props.children) && props.children));
}

function createBar(dimension, startLocation) {
  return _extends({
    id: makeId(),
    dimension: dimension,
    editing: true
  }, startLocation, {
    length: 1
  });
}
Reserver.defaultProps = {
  initBars: [],
  bars: [],
  tags: [],
  headRow: [],
  rowTitles: [],
  content: {},
  dimension: 20,
  width: 500,
  height: 500,
  rowTitleWidth: 0,
  mouseEnterCell: function mouseEnterCell() {},
  mouseDownCell: function mouseDownCell() {},
  mouseUpCell: function mouseUpCell() {},
  mouseDragOverCell: function mouseDragOverCell() {},
  mouseCellDrop: function mouseCellDrop() {},
  mouseLeaveGrid: function mouseLeaveGrid() {}
};

exports.Bar = Bar;
exports.actionTypes = actionTypes;
exports.createBar = createBar;
exports.default = Reserver;
exports.evaluatePosition = evaluatePosition;
exports.getPosition = getPosition;
exports.makeId = makeId;
exports.reserverReducer = reserverReducer;
exports.resizeBar = resizeBar;
exports.useReserver = useReserver;
//# sourceMappingURL=index.js.map
